'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

exports.default = function (types, filename) {
    var output = [];

    Object.keys(types).forEach(function (name) {
        var _types$name = types[name],
            flowtypes = _types$name.flowtypes,
            type = _types$name.type;

        if (flowtypes.length > 1) {
            flowtypes.forEach(function (flowtype, index) {
                output.push('export type ' + name + (index + 1) + ' = ' + (type === 'query' ? 'null | ' : '') + objectType(flowtype) + ';');
            });
        } else {
            output.push('export type ' + name + ' = ' + (type === 'query' ? 'null | ' : '') + objectType(flowtypes[0]) + ';');
        }
    });

    return '/* @flow\nDO NOT UPDATE THIS FILE MANUALLY\nAuto-generated flowtypes for Relay queries and fragments in ../' + filename + '\n*/\n\n' + output.join('\n\n') + '\n';
};

var INDENT = '    ';

;

function objectType(flowtype) {
    var indent = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

    if (!Object.keys(flowtype).length) {
        return '{||}';
    }
    return '{|\n' + Object.keys(flowtype).map(function (name) {
        return '' + indent + INDENT + name + ': ' + fieldType(flowtype[name], indent);
    }).join(',\n') + '\n' + indent + '|}';
}

function fieldType(field, indent) {
    var flowType;

    switch (field.type) {
        case 'list':
            flowType = 'Array<' + fieldType(field.ofType, indent) + '>';
            break;
        case 'object':
            flowType = objectType(field.object, '' + indent + INDENT);
            break;
        default:
            flowType = field.type;
            break;
    }

    if (field.nonNull) {
        return flowType;
    }
    return 'null | ' + flowType;
}