'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = function (contents, schema) {
    var flowtypes = {};

    // Remove comments
    contents = contents.replace(/\/\/.*/g, '').replace(/\/\*[.\s\S]*?\*\//g, '');
    var RelayQLRE = /Relay\.QL`([.\s\S]*?)`/g;
    var match;
    while (match = RelayQLRE.exec(contents)) {
        var originalQueryString = match[1];
        // Fix Relay's non-standard GraphQL
        var queryString = originalQueryString.replace('fragment on', 'fragment F1 on');
        // Remove comments
        queryString = queryString.replace(/#.*/g, ' ');
        // Remove newlines that may come from formatting and would break the regex below
        queryString = queryString.replace(/\n/g, ' ');
        queryString = queryString.replace(/\$\{[a-zA-Z0-9_]+}/g, PLACEHOLDER_FIELD_NAME + ': __typename');
        // Removing fragments could make an object empty so put a placeholder in there - __typename is something we know
        // will always be available
        queryString = queryString.replace(/\$\{.*?\.getFragment\(.*?\)}/g, PLACEHOLDER_FIELD_NAME + ': __typename');
        var parsed = (0, _parser.parse)(new _source.Source(queryString));
        var definition = parsed.definitions[0];
        switch (definition.kind) {
            case 'OperationDefinition':
                if (definition.operation === 'query') {
                    // There will only be one selection in a query
                    var selection = definition.selectionSet.selections[0];
                    if (selection.selectionSet && selection.selectionSet.selections.length) {
                        try {
                            var type = schema.getQueryType().getFields()[selection.name.value].type;
                            var flowtype = getObjectFlowType(selection, type, schema);
                            var typeName = type.name + 'QueryType';
                            flowtypes[typeName] = flowtypes[typeName] || { type: 'query', flowtypes: [] };
                            flowtypes[typeName].flowtypes.push(flowtype);
                        } catch (err) {
                            console.error('Could not handle Query\n\n' + originalQueryString);
                            throw err;
                        }
                    }
                }
                break;
            case 'FragmentDefinition':
                {
                    try {
                        var _type = schema.getType(definition.typeCondition.name.value);
                        var _flowtype = getObjectFlowType(definition, _type, schema);
                        var _typeName = _type.name + 'FragmentType';
                        flowtypes[_typeName] = flowtypes[_typeName] || { type: 'fragment', flowtypes: [] };
                        flowtypes[_typeName].flowtypes.push(_flowtype);
                    } catch (err) {
                        console.error('Could not handle Fragment\n\n' + originalQueryString);
                        throw err;
                    }
                    break;
                }
        }
    }

    return flowtypes;
};

var _parser = require('graphql/language/parser');

var _source = require('graphql/language/source');

var _definition = require('graphql/type/definition');

var PLACEHOLDER_FIELD_NAME = '___relayflowtypesplaceholder';

function getObjectFlowType(obj, type, schema) {
    var flowObj = {};
    if (obj.selectionSet) {
        obj.selectionSet.selections.forEach(function (selection) {
            var kind = selection.kind,
                alias = selection.alias;

            if (alias && alias.value === PLACEHOLDER_FIELD_NAME) {
                return;
            }

            switch (kind) {
                case 'Field':
                    {
                        var fieldName = selection.name.value;
                        var fieldAlias = selection.alias && selection.alias.value || fieldName;

                        switch (fieldName) {
                            case '__typename':
                                flowObj.__typename = { type: 'string', nonNull: true };
                                break;
                            default:
                                {
                                    var fieldType = type.getFields()[fieldName];
                                    flowObj[fieldAlias] = getFieldFlowType(selection, fieldType.type, schema);
                                    break;
                                }
                        }
                        break;
                    }
                case 'InlineFragment':
                    if (selection.typeCondition) {
                        var fragmentType = schema.getType(selection.typeCondition.name.value);
                        Object.assign(flowObj, getObjectFlowType(selection, fragmentType, schema));
                    }
                    break;
            }
        });
    }
    return flowObj;
}

function getFieldFlowType(field, type, schema) {
    if (type instanceof _definition.GraphQLNonNull) {
        return _extends({}, getFieldFlowType(field, type.ofType, schema), {
            nonNull: true
        });
    }

    if (type instanceof _definition.GraphQLList) {
        return {
            type: 'list',
            ofType: _extends({}, getFieldFlowType(field, type.ofType, schema), {
                nonNull: true
            })
        };
    }

    if ((0, _definition.isLeafType)(type)) {
        switch (type.name) {
            case 'Boolean':
                return { type: 'boolean' };
            case 'Int':
                return { type: 'number' };
            case 'Float':
                return { type: 'number' };
            default:
                // TODO: enums
                return { type: 'string' };
        }
    }

    if (type instanceof _definition.GraphQLObjectType || type instanceof _definition.GraphQLInterfaceType || type instanceof _definition.GraphQLUnionType) {
        return {
            type: 'object',
            object: getObjectFlowType(field, type, schema)
        };
    }

    throw Error('Unsupported field type ' + type.toString());
}